import fs from 'fs';
import { getCSSFilePath } from '@component-controls/core/node-utils';
import { asyncWebpackConfig } from '@component-controls/base-integration/webpack-build';

const HtmlWebpackPlugin = require('safe-require')('html-webpack-plugin');
class HTMLInjectStylePlugin {
    constructor(options) {
        this.options = options;
    }
    apply(compiler) {
        if (this.options.styles) {
            compiler.hooks.compilation.tap(HTMLInjectStylePlugin.pluginName, compilation => {
                if (HtmlWebpackPlugin && HtmlWebpackPlugin.getHooks) {
                    const hooks = HtmlWebpackPlugin.getHooks(compilation);
                    hooks.alterAssetTagGroups.tapAsync(HTMLInjectStylePlugin.pluginName, (data, cb) => {
                        data.headTags.push({
                            tagName: 'style',
                            innerHTML: this.options.styles,
                        });
                        cb(null, data);
                    });
                }
                else {
                    throw new Error('Cannot find HtmlWebpackPlugin hooks to inject css styles');
                }
            });
        }
    }
}
HTMLInjectStylePlugin.pluginName = 'html-inject-style-webpack-plugin';

const withComponentControls = (props) => async () => {
    return asyncWebpackConfig(Object.assign(Object.assign({}, props), { callback: async ({ config, options }) => {
            if (options) {
                const cssBundle = getCSSFilePath(options);
                if (fs.existsSync(cssBundle)) {
                    const styles = fs.readFileSync(cssBundle, 'utf8');
                    config.plugins.push(new HTMLInjectStylePlugin({ styles: styles }));
                    return config;
                }
            }
            return config;
        } }));
};

export { withComponentControls };
